@@ -4,12 +4,17 @@ use strict;
 use warnings;
 use Carp;
 
+use Dancer::Factory::Hook;
 use Dancer::Deprecation;
 use Dancer::FileUtils 'path';
 
 use base 'Dancer::Engine';
 
-# Overloads this method to implement the rendering
+Dancer::Factory::Hook->instance->install_hooks(
+    qw/before_template_render after_template_render before_layout_render after_layout_render/
+);
+
+# overloads this method to implement the rendering
 # args:   $self, $template, $tokens
 # return: a string of $template's content processed with $tokens
 sub render { confess "render not implemented" }
@@ -50,10 +55,12 @@ sub apply_renderer {
 
     $view = $self->view($view);
 
-    $_->($tokens) for (@{Dancer::App->current->registry->hooks->{before_template}});
+    Dancer::Factory::Hook->execute_hooks('before_template_render', $tokens);
 
     my $content = $self->render($view, $tokens);
 
+    Dancer::Factory::Hook->execute_hooks('after_template_render', \$content);
+    
     # make sure to avoid ( undef ) in list context return
     defined $content
       and return $content;
@@ -78,8 +85,13 @@ sub apply_layout {
 
     defined $layout or return $content;
 
+    Dancer::Factory::Hook->execute_hooks('before_layout_render', $tokens, \$content);
+
     my $full_content =
       $self->layout($layout, $tokens, $content);
+
+    Dancer::Factory::Hook->execute_hooks('after_layout_render', \$full_content);
+
     # make sure to avoid ( undef ) in list context return
     defined $full_content
       and return $full_content;
